# -*- coding: utf-8 -*-
import os
import sys

base_path=os.path.dirname(os.path.realpath(__file__))

defaultClientFolder = "/etc/pykmip"
defaultClientConfig = "/etc/pykmip/pykmip.conf"
host="15.207.252.192"
port="5696"
keyfile=os.path.join(base_path,"keys/client_key_jane_doe.pem")
certfile=os.path.join(base_path,"keys/client_certificate_jane_doe.pem")
ca_certs=os.path.join(base_path,"keys/root_certificate.pem")

if not os.path.exists(defaultClientFolder):
    os.makedirs(defaultClientFolder)

#if not os.path.exists(defaultClientConfig):
file = open(defaultClientConfig, 'w+')
file.write("""
[client]
host={}
port={}
keyfile={}
certfile={}
ca_certs={}
cert_reqs=CERT_REQUIRED
do_handshake_on_connect=True
suppress_ragged_eofs=True
username=example_username
password=example_password
""".format(host, port, keyfile, certfile, ca_certs))

file.close()
from kmip.pie import objects
from kmip.pie import client
from kmip.pie import exceptions
from kmip import enums
from kmip.pie.client import ProxyKmipClient
import sys
import traceback
import time
import db
from tabulate import tabulate

def progressbar(it, prefix="", size=60, file=sys.stdout):
    count = len(it)
    def show(j):
        x = int(size*j/count)
        file.write("%s[%s%s] %i/%i\r" % (prefix, "â–¶"*x, "."*(size-x), j, count))
        file.flush()        
    show(0)
    for i, item in enumerate(it):
        yield item
        show(i+1)
    file.write("\n")
    file.flush()

def login():
    print ''
    c.open()
    print "\nLog in sucessful."

def logout():
    c.close()
    print "\nLog out sucessful."

def checkKey(key):
    conn = db.create_connection(db.dbName)
    c = conn.cursor()
    query =c.execute("select * from mappings where system_id='{}'".format(key))
    names =c.fetchall()
    c.close()
    conn.close()
    if names:
        return False
    else:
        return True

def appendKeyToDB(key_id, system_id):
    conn = db.create_connection(db.dbName)
    c = conn.cursor()
    query = c.execute("insert into mappings(key_id,system_id) values(?,?)",[key_id,system_id])
    conn.commit()
    c.close()
    conn.close()

def addKey():
    key = str(raw_input("\nEnter a unique key id : "))
    key=key.strip()
    flag = checkKey(key)
    if not flag:
        print "\nKey id already exists, retry again"
        addKey()
        return False
    #with c:
    key_id = c.create(
        enums.CryptographicAlgorithm.AES,
        256,
        operation_policy_name='default',
        name=key,
        cryptographic_usage_mask=[
            enums.CryptographicUsageMask.ENCRYPT,
            enums.CryptographicUsageMask.DECRYPT
        ]
    )
    c.activate(key_id)
    appendKeyToDB(key_id,key)
    print "\n-----Added Key (using AES) => key_id : {}, user_key_id : {}-----".format(key_id,key)

def addKeyPair():
    pukey, prkey = str(raw_input("\nEnter a unique public & private key id : ")).split()
    pukey=pukey.strip()
    prkey=prkey.strip()
    if not checkKey(pukey) or not checkKey(prkey):
        print "\nKey id (one or both) already exists, retry again"
        addKeyPair()
        return False

    #with c:
    pukey_id, prkey_id = c.create_key_pair(
        enums.CryptographicAlgorithm.RSA,
        2048,
        operation_policy_name='default',
        public_name=pukey,
        public_usage_mask=[
            enums.CryptographicUsageMask.VERIFY
        ],
        private_name=prkey,
        private_usage_mask=[
            enums.CryptographicUsageMask.SIGN
        ]
    )
    c.activate(pukey_id)
    c.activate(prkey_id)
    appendKeyToDB(pukey_id,pukey)
    appendKeyToDB(prkey_id,prkey)
    print "\n-----Added Key (using RSA) => PUBLIC: key_id : {}, user_key_id : {}-----".format(pukey_id,pukey)
    print "-----                      => PRIVATE: key_id : {}, user_key_id : {}-----".format(prkey_id,prkey)

def getSystemIds():
    conn = db.create_connection(db.dbName)
    c = conn.cursor()
    data = []
    query = c.execute("select system_id from mappings")
    for indx,col in enumerate(c):
        data.append([int(indx+1),col[0]])
    c.close()
    conn.close()
    return data

def getSelectedKey(system_id):
    conn = db.create_connection(db.dbName)
    c = conn.cursor()
    data = None
    query = c.execute("select key_id from mappings where system_id='{}'".format(system_id))
    for col in c:
        data = col[0]
    c.close()
    conn.close()
    return data

def removeKey(key_id):
    conn = db.create_connection(db.dbName)
    c = conn.cursor()
    query = c.execute("delete from mappings where key_id={}".format(key_id))
    conn.commit()
    c.close()
    conn.close()

def getKeyFromSystemId():
    data = getSystemIds()
    if len(data) > 1:
        choice = int(raw_input("\nSelect a user_key_id ({}-{}): ".format(str(1),len(data))))
    else:
        choice = int(raw_input("\nSelect a unique user_key_id (1): "))
    if int(choice) not in range(1,len(data)+1):
        print "\nInvalid choice, select again "
        getKeyFromSystemId()
        return False
    selectedSystemId = data[choice-1][1]
    selectedKey = str(getSelectedKey(selectedSystemId))
    return selectedKey

def displayKeys():
    data = getSystemIds()
    if data:
        print ''
        #print(tabulate({'Keys':data}, headers='keys', tablefmt='pretty',colalign=("left",),showindex=range(1,len(data)+1)))
        print(tabulate(data, headers=['id','Keys'], tablefmt='pretty',colalign=("left","left")))
        return True
    else:
        print "\n----- No keys -----"
        return False

def displayKeyDetails():
    flag = displayKeys()
    if not flag:
        return 
    selectedKey = getKeyFromSystemId()
    data = []
    #with c:
    #attributes = ["Name",'Cryptographic Algorithm','Cryptographic Length','Object Type','State','Cryptographic Usage Mask','Initial Date','Operation Policy Name']
    attributes = c.get_attribute_list(selectedKey)
    attributes.sort()

    attribute_values=c.get_attributes(
        uid=selectedKey,
        attribute_names = attributes
    )
    for index,attr in enumerate(attribute_values[1]):
        data.append([attributes[index],attr.attribute_value])

    # val = c.get(selectedKey)
    # data.append(['Value',val.value])

    print "\nKey details\n"
    print(tabulate(data, headers=['Attribute','Value'], tablefmt='pretty',colalign=("left","left")))

    # print "\nKey Value : ",val.value

def invalid():
    print "\nINVALID CHOICE!"

c = ProxyKmipClient()
while True:
    try:
        menu = {
                    "1":("Log in",login),
                    "2":("Make a storage key",addKey),
                    "3":("Make a key-pair",addKeyPair),
                    "4":("Get a key",displayKeyDetails),
                    "5":("Logout",logout),
                    "999":("Quit",sys.exit)
            }

        print "\n========== MENU ==========" 
        for key in sorted(menu.keys()):
            print key+" \t- " + menu[key][0]

        ans = raw_input("\nMake A Choice : ")
        print "\n========== {} ==========".format(menu.get(ans,[None,None])[0]) 
        menu.get(ans,[None,invalid])[1]()

    except exceptions.ClientConnectionNotOpen as e:
        print "\n========== Pls log in first =========="
        
    except Exception as e:
        print "\nError occured : {}".format(str(e))
        print(traceback.format_exc())

    print ''
    for i in progressbar(range(10), "Displaying menu in: ",size=20):
        time.sleep(0.2)
